from enum import Enum


class DataType(Enum):
    YOUTUBE_VIDEO = "youtube_video"
    PDF_FILE = "pdf_file"
    WEB_PAGE = "web_page"
    SITEMAP = "sitemap"
    DOCX = "docx"
    DOCS_SITE = "docs_site"
    TEXT = "text"
    QNA_PAIR = "qna_pair"
    NOTION = "notion"
